@@ -278,7 +278,9 @@ QString KFormatPrivate::formatDecimalDuration(quint64 msecs, int decimalPlaces)
         return tr("%1 seconds").arg(m_locale.toString(msecs / (MSecsInSecond * 1.0), 'f',  decimalPlaces));
     }
     //: @item:intext %1 is a whole number
-    return tr("%1 millisecond(s)", 0, msecs).arg(msecs);
+    //~ singular %n millisecond
+    //~ plural %n milliseconds
+    return tr("%n millisecond(s)", 0, msecs);
 }
 
 enum DurationUnits {
@@ -294,15 +296,23 @@ static QString formatSingleDuration(DurationUnits units, int n)
     switch (units) {
     case Days:
         //: @item:intext %n is a whole number
+        //~ singular %n day
+        //~ plural %n days
         return KFormatPrivate::tr("%n day(s)", 0, n);
     case Hours:
         //: @item:intext %n is a whole number
+        //~ singular %n hour
+        //~ plural %n hours
         return KFormatPrivate::tr("%n hour(s)", 0, n);
     case Minutes:
         //: @item:intext %n is a whole number
+        //~ singular %n minute
+        //~ plural %n minutes
         return KFormatPrivate::tr("%n minute(s)", 0, n);
     case Seconds:
         //: @item:intext %n is a whole number
+        //~ singular %n second
+        //~ plural %n seconds
         return KFormatPrivate::tr("%n second(s)", 0, n);
     }
     Q_ASSERT(false);
